/**********************************************************************
 * 			MEKATRONIX Copyright 1998 		      *
 * Title        motortjp.c  (Motor Drivers for TJ Pro)                *
 * Programmer   Drew Bagnell modified TJ drivers                      *
 * Date         December, 1997                                        *
 * Version      1                                                     *
 * Version      1.1 by Keith L. Doty
     Changed some names, set PACTL bits with one command, cleaned up.
 *                                                                    *
 * Description                                                        *
 * Version History: These drivers follow Mechantronix TJP standard:   *
 * left motor is motor 0 (PA7) and right motor is motor 1 (PA3).      *
 * For efficient use of output compares, OC5 is used to modulated pins*
 * both PA3 and PA7. Changes were made to setup to allow drivers to   *
 * work with both A and E series MC68HC11 chips.		      *
 **********************************************************************/


/**************************** Includes **********************************/
#include <hc11.h>
#include <mil.h>
/************************************************************************/


/**************************** Constants *********************************/
#define HALFPERIOD 20000         /* 50Hz refresh for each motor */
#define MIN_PWM      2200         
#define MAX_PWM      3800
#define SPEED_SLOPE  8          /* modified for TJP */
#define SPEED_INT    3000

#define motortjp_hand   TOC5_isr
/************************************************************************/


/*************************** Prototypes *********************************/
#pragma interrupt_handler TOC5_isr
void init_motortjp(void);
void motorp(int, int);
void motortjp_hand();
/************************************************************************/


/***************************** Globals **********************************/
unsigned int width[2];       /* width[] is the on-time for each motor*/
unsigned int current_width;
char signal_state, motormask[2];
/************************************************************************/



void 
init_motortjp(void)
/************************************************************************
 * Function: Initializes all necessary variables for the motors.        *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:   width[], signal_state, motormask[], current_width       *
 *   Registers: TOC2, PORTB, TMSK1, TCTL1                               *
 * Functions called: None                                               *
 * Notes: None                                                          *
 ************************************************************************/
{

  INTR_OFF();
  CLEAR_BIT(TCTL1,0x03);          /* Interrupt will not affect OC5 pin */

  width[0] = width[1] =  0;      /* Motors start turned off */
  TOC5 = 0;                       
  signal_state = 0;
  current_width = 0;
  motormask[0] = 0x80;		/* Drives PA7, left motor*/
  motormask[1] = 0x08;		/* Drives PA3, right motor*/

  /* PA gates on (TJP specific) */
  /* Set PA3 (OC5) and PA7 (OC1) to output */ 
  SET_BIT(PACTL,0x88);  		

  SET_BIT(TMSK1,0x08);            /* Enable OC5 interrupt */
  INTR_ON();
}/*end init_motortjp*/


void
motorp(int index, int newwidth)
/************************************************************************
 * Function: Sets one of the motors to the speed defined by newwidth    *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: index, newwidth                                        *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals: width[index]                                              *
 *   Registers:  None                   		                *
 * Functions called: None                                               *
 * Notes: None                                                          *
 ************************************************************************/
{
  /* Allow both "natural" motor (-100 <--> 100) or motor pw */
  if ( newwidth < 2000 && newwidth)
  {
  	if (!index) 
  		newwidth = (-(newwidth*SPEED_SLOPE) + SPEED_INT);
  	else
  		newwidth = ((newwidth*SPEED_SLOPE) + SPEED_INT);
  }

  /* Load newwidth into width[index]*/
  asm("ldd %index \n"
      "aslb \n"
      "ldy #_width \n"
      "aby \n"
      "ldd %newwidth \n"
      "std 0,y");

}/*end motorp*/



void motortjp_hand(void)
/************************************************************************
 * Function: Interrupt handler for motor signals                        *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    width[], signal_state, motormask[], current_width      *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    current_width                                          *
 *   Registers:  TOC5, PORTA, TFLG1      				                *
 * Functions called: None                                               *
 * Notes: None                                                          *
 ************************************************************************/
{
  char odd, index;
  unsigned int pwidth;

/*
 Control is passed alternately between the motors each half period
   signal_state = 0  -> Turn on motor0
   signal_state = 1  -> Turn off motor0
   signal_state = 2  -> Turn on motor1
   signal_state = 3  -> Turn off motor1
*/

  signal_state &= 0x03;         /* Only use last 2 bits */
  index = (signal_state >> 1);  /* index references current motor */

  odd = (signal_state & 0x01);

  pwidth = *(width+index); /*Load the newwidth into pwidth*/
  
  if ((pwidth == 0)&&(!(odd)))
  {
    TOC5 += HALFPERIOD;
    signal_state++;
  }
  else
  {
    if (odd)
      TOC5 += (HALFPERIOD - current_width);
    else
      TOC5 += pwidth;

  /* Right motor signal goes on PA3, and left motor signal goes on PA7*/
    PORTA ^= *(motormask+index);
  }

  current_width = pwidth;
  signal_state++;
  if (signal_state == 4)
    signal_state = 0;
   
  CLEAR_FLAG(TFLG1,0x08);          /* Clear OC5I flag */

}/*end of motortjp_hand*/
